// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include "brushwidget.h"

BrushWidget::BrushWidget(QWidget *parent):QWidget(parent)
{


}

BrushWidget::~BrushWidget()
{

}

